package algorithms;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import objects.DistanceMatrix;
import objects.Template;

public class ClassifierKNNMV extends Classifier {

    private final int k;

    public ClassifierKNNMV(int k) {
        this.k = k;
    }

    @Override
    public int classify(Template templateQuery, List<Template> templatesGallery) {

        // get k closest descriptors
        Map<Double, Template> kClosestTemplates = new HashMap(k);
        DistanceTemplates distanceTemplates = getDistanceTemplates();
        DistanceMatrix distanceMatrix = getDistanceMatrix();
        double maxDistance = 0;
        for (Template template : templatesGallery) {
            double distance = getDistanceMatrix() == null ? distanceTemplates.getDistance(template, templateQuery) : distanceMatrix.getDistance(template, templateQuery);
            if (kClosestTemplates.size() < k || distance < maxDistance) {
                kClosestTemplates.put(distance, template);
                if (kClosestTemplates.size() > k) {
                    kClosestTemplates.remove(maxDistance);
                }
            }
            Iterator<Double> iterator = kClosestTemplates.keySet().iterator();
            maxDistance = 0;
            while (iterator.hasNext()) {
                Double key = iterator.next();
                if (maxDistance < key) {
                    maxDistance = key;
                }
            }
        }

        // majority voting
        List<List<Integer>> votes = new ArrayList();
        Iterator<Double> iterator = kClosestTemplates.keySet().iterator();
        while (iterator.hasNext()) {
            Template template = kClosestTemplates.get(iterator.next());
            boolean voteIsInVotes = false;
            for (List<Integer> vote : votes) {
                if (vote.get(0) == template.getSubject()) {
                    voteIsInVotes = true;
                    vote.set(1, vote.get(1) + 1);
                    break;
                }
            }
            if (!voteIsInVotes) {
                List<Integer> vote = new ArrayList();
                vote.add(template.getSubject());
                vote.add(1);
                votes.add(vote);
            }
        }
        int maxNumberVotes = 0;
        int winSubject = 0;
        for (List<Integer> vote : votes) {
            int numberVotes = vote.get(1);
            if (maxNumberVotes < numberVotes) {
                maxNumberVotes = numberVotes;
                winSubject = vote.get(0);
            }
        }
        return winSubject;
    }
}
