package methods;

import algorithms.Classifier;
import algorithms.ClassifierKNNMV;
import algorithms.DistanceTemplatesDTW;
import algorithms.DistanceVectors;
import algorithms.DistanceVectorsL1;
import algorithms.RetrieverKNNQuery;
import executor.Executor;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import objects.Feature;
import objects.Motion;
import objects.MotionJointCoordinates;
import objects.Template;

public class MethodGavrilova extends Method {

    public MethodGavrilova() {
    }

    @Override
    public Motion loadMotion(File fileAMC) throws IOException {
        return Executor.loadMotionJointCoordinates(fileAMC);
    }

    @Override
    public void learnClassifier(List<Motion> motionsLearning) {
        ClassifierGavrilova classifier = new ClassifierGavrilova();
        DistanceTemplatesDTW distanceTemplatesJRD = new DistanceTemplatesDTW();
        DistanceTemplatesDTW distanceTemplatesJRA = new DistanceTemplatesDTW();
        DistanceVectorsGavrilovaJRD distanceVectorsJRD = new DistanceVectorsGavrilovaJRD();
        DistanceVectorsGavrilovaJRA distanceVectorsJRA = new DistanceVectorsGavrilovaJRA();
        distanceTemplatesJRD.setDistanceVectors(distanceVectorsJRD);
        distanceTemplatesJRA.setDistanceVectors(distanceVectorsJRA);
        classifier.setDistanceTemplatesJRD(distanceTemplatesJRD);
        classifier.setDistanceTemplatesJRA(distanceTemplatesJRA);
        DistanceTemplatesDTW distanceTemplates = new DistanceTemplatesDTW();
        distanceTemplates.setDistanceVectors(new DistanceVectorsL1());
        classifier.setDistanceTemplates(distanceTemplates);
        setClassifier(classifier);
    }

    @Override
    public List<Feature> extractFeatures(Motion motion) { // JRD+JRA
        MotionJointCoordinates motionMJC = (MotionJointCoordinates) motion;
        List<Feature> features = new ArrayList();
        features.add(motionMJC.extractInterJointDistanceFeature("lowerneck", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("lowerneck", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("lowerneck", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("lowerneck", "rtibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("lclavicle", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("lclavicle", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("rclavicle", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("rclavicle", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("lclavicle", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("lclavicle", "rtibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("rclavicle", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("rclavicle", "rtibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("thorax", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("thorax", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("thorax", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("thorax", "rtibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("root", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("root", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("root", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("root", "rtibia"));
        features.add(motionMJC.extractJointAngleFeature("lclavicle", "root", "lfemur"));
        features.add(motionMJC.extractJointAngleFeature("lclavicle", "root", "rfemur"));
        features.add(motionMJC.extractJointAngleFeature("rclavicle", "root", "lfemur"));
        features.add(motionMJC.extractJointAngleFeature("rclavicle", "root", "rfemur"));
        features.add(motionMJC.extractJointAngleFeature("lclavicle", "root", "ltibia"));
        features.add(motionMJC.extractJointAngleFeature("lclavicle", "root", "rtibia"));
        features.add(motionMJC.extractJointAngleFeature("rclavicle", "root", "ltibia"));
        features.add(motionMJC.extractJointAngleFeature("rclavicle", "root", "rtibia"));
        features.add(motionMJC.extractJointAngleFeature("lhipjoint", "root", "lfemur"));
        features.add(motionMJC.extractJointAngleFeature("lhipjoint", "root", "rfemur"));
        features.add(motionMJC.extractJointAngleFeature("rhipjoint", "root", "lfemur"));
        features.add(motionMJC.extractJointAngleFeature("rhipjoint", "root", "rfemur"));
        features.add(motionMJC.extractJointAngleFeature("lhipjoint", "root", "ltibia"));
        features.add(motionMJC.extractJointAngleFeature("lhipjoint", "root", "rtibia"));
        features.add(motionMJC.extractJointAngleFeature("rhipjoint", "root", "ltibia"));
        features.add(motionMJC.extractJointAngleFeature("rhipjoint", "root", "rtibia"));
        return features;
    }

    public class ClassifierGavrilova extends Classifier {

        private final int k = 3; // 3-NN
        private DistanceTemplatesDTW distanceFunctionJRD;
        private DistanceTemplatesDTW distanceFunctionJRA;

        public ClassifierGavrilova() {
        }

        public void setDistanceTemplatesJRD(DistanceTemplatesDTW distanceFunctionJRD) {
            this.distanceFunctionJRD = distanceFunctionJRD;
        }

        public void setDistanceTemplatesJRA(DistanceTemplatesDTW distanceFunctionJRA) {
            this.distanceFunctionJRA = distanceFunctionJRA;
        }

        @Override
        public int classify(Template querySample, List<Template> trainingSamples) {
            RetrieverKNNQuery retrieverJRD = new RetrieverKNNQuery(k);
            RetrieverKNNQuery retrieverJRA = new RetrieverKNNQuery(k);
            retrieverJRD.setDistanceTemplates(distanceFunctionJRD);
            retrieverJRA.setDistanceTemplates(distanceFunctionJRA);
            List<Template> retrievedJRD = retrieverJRD.retrieve(querySample, trainingSamples);
            List<Template> retrievedJRA = retrieverJRA.retrieve(querySample, trainingSamples);
            List<Template> retrieved = new ArrayList();
            retrieved.addAll(retrievedJRD);
            retrieved.addAll(retrievedJRA);
            ClassifierKNNMV classifier = new ClassifierKNNMV(2 * k);
            DistanceTemplatesDTW distanceFunction = new DistanceTemplatesDTW();
            distanceFunction.setDistanceVectors(new DistanceVectorsL1());
            classifier.setDistanceTemplates(distanceFunction);
            return classifier.classify(querySample, retrieved);
        }
    }

    public final class DistanceVectorsGavrilovaJRD extends DistanceVectors {

        public DistanceVectorsGavrilovaJRD() {
        }

        @Override
        public double getDistance(double[] pose1, double[] pose2) {
            if (pose1.length != pose2.length) {
                System.out.println("wrong dimensions");
            }
            double distance = 0;
            for (int d = 0; d < pose1.length; d++) {
                if (d <= 20) {
                    distance += Math.abs(pose1[d] - pose2[d]);
                }
            }
            return distance;
        }
    }

    public final class DistanceVectorsGavrilovaJRA extends DistanceVectors {

        public DistanceVectorsGavrilovaJRA() {
        }

        @Override
        public double getDistance(double[] pose1, double[] pose2) {
            if (pose1.length != pose2.length) {
                System.out.println("wrong dimensions");
            }
            double distance = 0;
            for (int d = 0; d < pose1.length; d++) {
                if (d >= 21) {
                    distance += Math.abs(pose1[d] - pose2[d]);
                }
            }
            return distance;
        }
    }
}
